﻿using System;

namespace GameUtils.Audio
{
    public abstract class AudioHandle : IDisposable
    {
        public event EventHandler PlaybackStopped;

        public abstract TimeSpan Length { get; }

        public abstract TimeSpan Position { get; set; }

        public abstract float Volume { get; set; }

        public abstract PlaybackState State { get; }

        public abstract void Play();

        public abstract void Pause();

        public abstract void Stop();

        protected virtual void OnPlaybackStopped(EventArgs e)
        {
            if (PlaybackStopped != null)
                PlaybackStopped(this, e);
        }

        private bool disposed;

        public void Dispose()
        {
            if (!disposed)
            {
                Dispose(true);
                GC.SuppressFinalize(this);

                disposed = true;
            }
        }

        protected virtual void Dispose(bool disposing)
        { }

        ~AudioHandle()
        {
            Dispose(false);
        }
    }
}
