﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using SharpDX.DirectInput;

namespace GameUtils.Input.DefaultDevices
{
    public class Keyboard : InputDevice<KeyboardState>
    {
        DirectInput input;
        SharpDX.DirectInput.Keyboard keyboard;
        AutoResetEvent eventHandle;
        Thread t;
        bool capture;
        LinkedList<IInputListener<KeyboardState>> listeners;

        public Keyboard()
        {
            listeners = new LinkedList<IInputListener<KeyboardState>>();
        }

        public override ListenerHandle<KeyboardState> RegisterListener(IInputListener<KeyboardState> listener)
        {
            LinkedListNode<IInputListener<KeyboardState>> node = listeners.AddLast(listener);
            return new ListenerHandle<KeyboardState>(listeners, node);
        }

        public override void Initialize(System.Windows.Forms.Control surface)
        {
            input = new DirectInput();
            keyboard = new SharpDX.DirectInput.Keyboard(input);
            keyboard.SetCooperativeLevel(surface, CooperativeLevel.Background | CooperativeLevel.NonExclusive);
        }

        public override void BeginCapture()
        {
            eventHandle = new AutoResetEvent(false);
            keyboard.SetNotification(eventHandle);
            keyboard.Acquire();
            capture = true;
            t = new Thread(() =>
            {
                while (capture)
                {
                    eventHandle.WaitOne();
                    if (capture)
                    {
                        SharpDX.DirectInput.KeyboardState stateDI = keyboard.GetCurrentState();
                        KeyboardState state = new KeyboardState(stateDI.PressedKeys.Select((key, index) => (Key)key).ToArray());
                        foreach (IInputListener<KeyboardState> listener in listeners)
                        {
                            listener.OnInputReceived(state);
                        }
                    }
                }
            });
            t.Start();
        }

        public override void EndCapture()
        {
            capture = false;
            eventHandle.Set();
            t.Join();
            keyboard.Unacquire();
            eventHandle.Dispose();
            eventHandle = null;
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                listeners = null;
                if (eventHandle != null) eventHandle.Dispose();
            }

            keyboard.Dispose();
            input.Dispose();
        }
    }
}
