﻿using System;
using GameUtils.Graphics.Data;
using SharpDX;
using SharpDX.Direct2D1;
using Vector2 = GameUtils.Math.Vector2;

namespace GameUtils.Renderers.Direct2D1
{
    internal class Direct2DLinearGradientBrush : LinearGradientBrushData, IDirect2DBrush
    {
        LinearGradientBrush brush;
        Graphics.GradientStop[] stops;
        Matrix3x2 matrix;
        readonly RenderTarget target;

        public Brush Brush
        {
            get { return brush; }
        }

        protected override Graphics.GradientStop[] GradientStops
        {
            get { return stops; }
            set
            {
                if (value == null)
                    throw new ArgumentNullException("value");
                if (value.Length < 2)
                    throw new ArgumentException("Es müssen mindestens zwei Verlaufsfarben angegeben sein.", "value");

                stops = value;
                var props = new LinearGradientBrushProperties();
                props.StartPoint = brush.StartPoint;
                props.EndPoint = brush.EndPoint;
                brush.Dispose();
                brush = new LinearGradientBrush(target, props, new GradientStopCollection(target, Direct2DHelper.ConvertStops(stops)));
                brush.Transform = matrix;
            }
        }

        protected override Vector2 StartPoint
        {
            get { return Direct2DHelper.PointFToVector2(brush.StartPoint); }
            set { brush.StartPoint = Direct2DHelper.Vector2ToPointF(value); }
        }

        protected override Vector2 EndPoint
        {
            get { return Direct2DHelper.PointFToVector2(brush.EndPoint); }
            set { brush.EndPoint = Direct2DHelper.Vector2ToPointF(value); }
        }

        protected override float Opacity
        {
            get { return brush.Opacity; }
            set { brush.Opacity = value; }
        }

        internal Direct2DLinearGradientBrush(RenderTarget target, Graphics.GradientStop[] stops, Vector2 start, Vector2 end)
        {
            matrix = Matrix3x2.Identity;
            this.target = target;
            this.stops = stops;
            var props = new LinearGradientBrushProperties();
            props.StartPoint = Direct2DHelper.Vector2ToPointF(start);
            props.EndPoint = Direct2DHelper.Vector2ToPointF(end);
            brush = new LinearGradientBrush(target, props, new GradientStopCollection(target, Direct2DHelper.ConvertStops(stops)));
        }

        protected override void SetTransform(Math.Matrix2x3 matrix)
        {
            var d2dMatrix = new Matrix3x2();
            d2dMatrix.TranslationVector = new SharpDX.Vector2(matrix.OffsetX, matrix.OffsetY);
            d2dMatrix.ScaleVector = new SharpDX.Vector2(matrix[0, 0], matrix[1, 1]);
            d2dMatrix.M12 = matrix[1, 0];
            d2dMatrix.M21 = matrix[0, 1];
            brush.Transform = d2dMatrix;
            this.matrix = d2dMatrix;
        }

        protected override void ResetTransform()
        {
            brush.Transform = Matrix3x2.Identity;
            matrix = Matrix3x2.Identity;
        }

        protected override void Dispose(bool disposing)
        {
            brush.Dispose();
        }
    }
}
