﻿using System.Drawing;
using GameUtils.Graphics;
using GameUtils.Graphics.Data;

namespace GameUtils.Renderers.Gdi
{
    internal class GdiFont : FontData
	{
		System.Drawing.Font font;
		string familyName;
		float size;
		System.Drawing.FontStyle style;

		public System.Drawing.Font GetFont()
		{
			return font;
		}

		protected override string FamilyName
		{
			get { return familyName; }
			set
			{
				familyName = value;
				font.Dispose();
                font = new System.Drawing.Font(familyName, size, this.style, GraphicsUnit.Pixel);
			}
		}

        protected override float Size
		{
			get { return size; }
			set
			{
				size = value;
				font.Dispose();
                font = new System.Drawing.Font(familyName, size, this.style, GraphicsUnit.Pixel);
			}
		}

        protected override Graphics.FontStyle Style
		{
			get
			{
				if ((style & System.Drawing.FontStyle.Italic) == System.Drawing.FontStyle.Italic)
					return Graphics.FontStyle.Italic;
				else
					return Graphics.FontStyle.Normal;
			}
			set
			{
				style = GdiHelper.GetFontStyle(value, Weight);
				font.Dispose();
                font = new System.Drawing.Font(familyName, size, this.style, GraphicsUnit.Pixel);
			}
		}

        protected override FontWeight Weight
		{
			get
			{
				if ((style & System.Drawing.FontStyle.Bold) == System.Drawing.FontStyle.Bold)
					return FontWeight.Bold;
				else
					return FontWeight.Normal;
			}
			set
			{
				style = GdiHelper.GetFontStyle(Style, value);
				font.Dispose();
                font = new System.Drawing.Font(familyName, size, this.style, GraphicsUnit.Pixel);
			}
		}

		internal GdiFont(string familyName, float size, Graphics.FontStyle style, FontWeight weight)
		{
			this.familyName = familyName;
			this.size = size;
			this.style = GdiHelper.GetFontStyle(style, weight);
            font = new System.Drawing.Font(familyName, size, this.style, GraphicsUnit.Pixel);
		}

		protected override void Dispose(bool disposing)
		{
			font.Dispose();
		}
	}
}
