﻿using System.Drawing;
using GameUtils.Graphics;
using GameUtils.Graphics.Data;

namespace GameUtils.Renderers.Gdi
{
    internal class GdiSolidColorBrush : SolidColorBrushData, IGdiBrush
	{
		readonly SolidBrush brush;
		Color4 color;
		float opacity;

        public Brush Brush
        {
            get { return brush; }
        }

        protected override Color4 Color
		{
			get { return color; }
			set
			{
				color = value;
				SetBrushColor(color, opacity);
			}
		}

        protected override float Opacity
		{
			get { return opacity; }
			set
			{
				if (value > 1)
					opacity = 1;
				else if (value < 0)
					opacity = 0;
				else
					opacity = value;
				SetBrushColor(color, opacity);
			}
		}

		private void SetBrushColor(Color4 baseColor, float opacity)
		{
			baseColor.A *= opacity;
			brush.Color = GdiHelper.ConvertColor(baseColor);
		}

		internal GdiSolidColorBrush(Color4 color)
		{
			opacity = 1;
		    this.color = color;
			brush = new SolidBrush(GdiHelper.ConvertColor(color));
		}

        protected override void SetTransform(Math.Matrix2x3 matrix)
        { }

        protected override void ResetTransform()
        { }

		protected override void Dispose(bool disposing)
		{
			brush.Dispose();
		}
    }
}
