﻿using System;

namespace GameUtils.Graphics.Data
{
    public abstract class ResourceData : IDisposable
    {
        /// <summary>
        /// Extracts the underlying data of a resource.
        /// </summary>
        /// <param name="resource">The resource to extract the data from.</param>
        protected T GetResourceData<T>(Resource<T> resource) where T : ResourceData
        {
            return resource.Data;
        }

        private bool disposed;

        public void Dispose()
        {
            if (!disposed)
            {
                Dispose(true);
                GC.SuppressFinalize(this);

                disposed = true;
            }
        }

        protected virtual void Dispose(bool disposing)
        { }

        ~ResourceData()
        {
            Dispose(false);
        }
    }
}
