﻿using System;
using GameUtils.Graphics.Data;

namespace GameUtils.Graphics
{
    public class TextureBrush : Brush<TextureBrushData>
    {
        Texture texture;
        TextureWrapMode wrapMode;
        bool textureChanged;
        bool wrapModeChanged;
        readonly object locker;

        /// <summary>
        /// The texture used in this texture brush.
        /// </summary>
        public Texture Texture
        {
            get { return texture; }
            set
            {
                if (value == null)
                    throw new ArgumentNullException("value");

                lock (locker)
                {
                    texture = value;
                    textureChanged = true;
                }
            }
        }

        /// <summary>
        /// Indicates how the texture should be wrapped.
        /// </summary>
        public TextureWrapMode WrapMode
        {
            get { return wrapMode; }
            set
            {
                lock (locker)
                {
                    wrapMode = value;
                    wrapModeChanged = true;
                }
            }
        }

        /// <summary>
        /// Creates a new texture brush.
        /// </summary>
        public TextureBrush(Texture texture)
        {
            if (texture == null)
                throw new ArgumentNullException("texture");

            this.texture = texture;
            locker = new object();
        }

        protected override void ApplyChanges()
        {
            lock (locker)
            {
                if (textureChanged)
                {
                    Data.Texture = texture;
                    textureChanged = false;
                }
                if (wrapModeChanged)
                {
                    Data.WrapMode = wrapMode;
                    wrapModeChanged = false;
                }
            }

            base.ApplyChanges();
        }

        protected override TextureBrushData CreateData(Factory factory)
        {
            lock (locker)
            {
                textureChanged = false;
                wrapModeChanged = false;
                return factory.CreateTextureBrush(texture);
            }
        }
    }
}
