﻿using System;
using GameUtils;
using GameUtils.Graphics;
using GameUtils.Math;

namespace TestApp
{
    public class GeometryTester : RegistrationContext<GeometryTester.GeometryTesterUpdateable>
    {
        readonly GeometryTesterRenderable renderable;

        public GeometryTester()
        {
            renderable = new GeometryTesterRenderable();
        }

        public class GeometryTesterRenderable : IStateRenderer<GeometryTesterUpdateable>
        {
            readonly SolidColorBrush brush;

            public GeometryTesterRenderable()
            {
                brush = new SolidColorBrush(Color4.White);
            }

            public void Render(GeometryTesterUpdateable state, Renderer renderer)
            {
                renderer.DrawGeometry(state.Geometry, brush, 3);
            }

            public event EventHandler DepthPositionChanged;

            float IStateRenderer<GeometryTesterUpdateable>.DepthPosition
            {
                get { return 0; }
            }
        }

        public class GeometryTesterUpdateable : IBufferedState<GeometryTesterUpdateable>
        {
            public Geometry Geometry { get; private set; }

            public Matrix2x3 Matrix { get; private set; }

            public GeometryTesterUpdateable()
            {
                Geometry = new Geometry();
                Geometry.AddRectangle(new Rectangle(150, 150, 300, 300));
                Geometry.Close();
                Matrix = Matrix2x3.Identity;
            }

            public void Update(GeometryTesterUpdateable oldState, TimeSpan elapsed)
            {
                Matrix = Matrix2x3.Translation(300, 300) * Matrix2x3.Rotation(3f * (float)elapsed.TotalSeconds) * Matrix2x3.Translation(-300, -300);
                Geometry.Transform(oldState.Matrix * Matrix);
            }
        }

        protected override GeometryTesterUpdateable CreateBuffer()
        {
            return new GeometryTesterUpdateable();
        }

        protected override IStateRenderer<GeometryTesterUpdateable> Renderer
        {
            get { return renderable; }
        }
    }
}
